/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __operate_OperateCommon_h__
#define __operate_OperateCommon_h__

namespace fe
{

/**************************************************************************//**
    @brief Common Operator/Manipulator methods

	@ingroup operate
*//***************************************************************************/
class FE_DL_EXPORT OperateCommon:
	public Catalog,
	virtual public BindI
{
	public:
		enum	CurveMode
				{
					e_null,
					e_lines,
					e_panels,
					e_cylinders,
					e_tube
				};

							OperateCommon(void)								{}
virtual						~OperateCommon(void)							{}

							//* As BindI
virtual	void				bind(sp<SignalerI> a_spSignalerI,
									sp<Layout> a_spLayout);

		Record				windowEvent(Record& a_rSignal)
							{
								Record* pRecord=
										m_aWindowEvent.queryAttribute(
										a_rSignal);
								if(pRecord)
								{
									return *pRecord;
								}
								return Record();
							}

static	void				drawLabel(sp<DrawI> a_spDrawI,
									const SpatialVector& a_location,
									String a_label,
									BWORD a_centered,
									U32 a_margin,
									const Color a_textColor,
									const Color* a_pBorderColor,
									const Color* a_pBackground);
static	void				drawPicker(sp<DrawI> a_spDrawI,
									const SpatialVector& a_point,
									const SpatialVector& a_up,
									const SpatialVector& a_normal,
									Real a_scale,I32 a_depth,
									const Color* a_pColor);
static	void				drawImpact(sp<DrawI> a_spDrawI,
									sp<SurfaceTriangles::Impact> a_spTriImpact,
									Real a_inset,const Color* a_pColor);
static	void				drawInsetTriangle(sp<DrawI> a_spDrawI,
									const SpatialVector& a_vertex0,
									const SpatialVector& a_vertex1,
									const SpatialVector& a_vertex2,
									Real a_inset,const Color* a_pColor);
static	void				highlightCurve(sp<ViewI> a_spView,
									sp<DrawI> a_spDraw,
									sp<SurfaceAccessorI> a_spVertices,
									I32 a_primitiveIndex,I32 a_vertexIndex,
									const Color* a_pLineColor,
									const Color* a_pPointColor,
									const Color* a_pPointHighlight);
static	void				drawDot(sp<ViewI> a_spView,sp<DrawI> a_spDraw,
									SpatialVector a_point,Real a_radius,
									const Color a_color);
static	void				drawSquare(sp<DrawI>& a_rspDraw,
									const SpatialVector& a_rPoint,
									Real a_rimSize,const Color& a_rColorCenter,
									const Color& a_rColorRim,
									const Color* a_pColorShadow,
									const Color* a_pColorSparkle);
static	void				drawTube(sp<DrawI>& a_rspDraw,
									const SpatialVector* a_point,
									const SpatialVector* a_normal,
									const Real* a_radius,
									U32 a_pointCount,CurveMode a_curveMode,
									BWORD a_multicolor,const Color *a_color,
									const SpatialVector& a_rTowardCamera);
static	void				drawTubeAxes(sp<DrawI>& a_rspDraw,
									const SpatialTransform& a_transform,
									Real a_scale,Real a_alpha,
									const std::vector<Real>& a_profile,
									CurveMode a_curveMode);
static	void				drawAnchor(sp<DrawI>& a_rspDraw,
									SpatialTransform a_transform,
									Real a_scale,U32 a_resolution,
									Color a_color);

static	I32					nearestPointIndex(
									const sp<SurfaceAccessorI>
									a_spPrimitiveVertices,
									sp<SurfaceCurves::Impact>&
									a_rspCurveImpact);

static	I32					nearestTriVertexIndex(
									sp<SurfaceTriangles::Impact>&
									a_rspTriImpact);
static	I32					nearestPointIndex(sp<SurfaceTriangles::Impact>&
									a_rspTriImpact);
static	SpatialVector		nearestVertexPoint(sp<SurfaceTriangles::Impact>&
									a_rspTriImpact);

static	Real				nearestPointAlongCurve(const SpatialVector a_point,
									const sp<SurfaceAccessorI>
									a_spPrimitiveVertices,
									const I32 a_primitiveIndex);

	private:

		Accessor<Record>	m_aWindowEvent;
};

} // namespace

#endif /* __operate_OperateCommon_h__ */
